MODULE module_noahlsm_dry_dep

    USE module_wddm
    USE module_dry_dep_zhang 

    CONTAINS

 SUBROUTINE dep_driver1(iyear,julday,ivegtyp,numgas,z,d,rdtheta,t,rad,RHf,prcp,cmc,ustar,z0,rmol,Rs,  & ! I
                       ra, rb, rc, vd)                                                                 ! O
! This subroutine calculates Vd using Rns from Wesely 1989.

    IMPLICIT NONE
    INTEGER :: iyear,julday, numgas, ivegtyp
    REAL :: z, d, rdtheta
    REAL :: t,rad,RHf,prcp,cmc
    REAL :: ustar,z0,rmol
    REAL, DIMENSION(numgas) :: hstar, f0, Dratio,dhr, sc
    INTEGER :: n, iseason, iland, p_so2, p_o3
    LOGICAL :: wetflag, dewflag, rainflag
    REAL :: Pr
    INTEGER :: mapusgs(25)
    real, parameter :: karman=0.40
    REAL :: rs
    REAL, DIMENSION(numgas) :: rb, rc, vd
    REAL :: ra
    INTEGER :: LB

    DATA mapusgs/1, 2, 2, 2, 2, 4, 3, 3, 3, 3, 4, 5, 4, 5, 6, 7, 9, 6, 8, 9, 6, 6, 8, 0, 0/

!-- Determin season and landuse type in Wesely's category    
    iland = mapusgs(ivegtyp)

!-- Compute leapyear bias (1 or 0), and season.

    IF(MOD(iyear,4).EQ.0) THEN
       LB=1
    ELSE
       LB=0
    END IF

    IF(julday.GE.(152+LB).AND.julday.LT.(244+LB)) THEN   
       iseason = 1        ! Midsummer with lush vegetation: Jun - Aug
    ELSE IF(julday.GE.(244+LB).AND.julday.LT.(305+LB)) THEN
       iseason = 2        ! Autumn with unharvested cropland: Sep - Oct
    ELSE IF(julday.GE.(305+LB).AND.julday.LT.(335+LB)) THEN
       iseason = 3        ! late Autumn after frost but no snow: Nov 
    ELSE IF(julday.GE.(335+LB).OR.julday.LT.(60+LB)) THEN
       iseason = 4        ! Winter with snow on the ground: Dec - Feb
    ELSE IF(julday.GE.(60+LB).AND.julday.LT.(152+LB)) THEN
       iseason = 5        ! transitional Spring with partial green coverage: Mar - May
    END IF   

    wetflag =.FALSE.
    dewflag = .FALSE.
    rainflag = .FALSE.
    if (cmc .gt. 0.)    wetflag = .true.
    if (RHf .ge. 0.95)  dewflag = .true.
    if (prcp.gt.0.)     rainflag = .true.    

    CALL dep_init(numgas,Hstar,f0,Dratio,dhr,sc,p_so2,p_o3)  

    Pr=0.72
    CALL calrb(numgas,sc,pr,ustar,karman,rb)

    d = 0.
    rdtheta = 10./180.*3.1415926

!    CALL calrc(Rs,t,rad,rdtheta,hstar,f0,Dratio,dhr,iland,iseason,wetflag,dewflag,rainflag,numgas,p_so2,p_o3,Rc)

    DO n = 1, numgas
        vd(n) = 100./(ra+rb(n)+rc(n))
    END DO 

 END SUBROUTINE dep_driver1

 SUBROUTINE dep_driver2( latitude, julday, ihour, zlvl_wind, vegtyp,XLAI,         & 
                         Ra, Rs,                                                  &
                         SFCTMP, T1, SOLDN, SFCSPD, SFCPRS, PRCP, RHf,USTAR,CMC,DEW,  &
                         Rb,Rm, Rac, Rg, Rcut, Rns, Wst, Rc, Vd, Rs_block, Dratio )
   implicit none
   INTEGER :: NG, NP, luc, NSIZE
   PARAMETER (NG=31)                ! number of gaseous species
   PARAMETER (NP=14)                ! number of particle species
   PARAMETER (luc=26)       ! NUMBER OF LAND-USE CATEGORIES

   REAL :: latitude, zlvl_wind, XLAI
   REAL :: SFCTMP, T1, SOLDN, SFCSPD, SFCPRS, PRCP, RHf, USTAR, CMC, DEW
   REAL :: Ra, Rs
   INTEGER :: julday, ihour, vegtyp   

   REAL :: VDG(NG), VDP(NP)      
   REAL :: Ra_z, Rac, Rs_z
   REAL, DIMENSION(NG) :: Rb, Rg, Rcut, Rns, Rc_z, Dratio, Rm, Vd, Rc  
   
! local variable
   real :: sd, FCLD, prec, pmb, COSZEN, Wst, Rs_block
   INTEGER :: j
   LOGICAL :: wetflag, rainflag, dewflag
   real, dimension(luc) :: fland


    sd = 0.0 ! snow depth
    FCLD = 0.2 ! Cloud fraction (0.0-1.0)
    prec = PRCP * 3600 ! kg m{-2} s{-1} --> mm/hour
    pmb = SFCPRS * 0.01 ! Pa --> mb
  
    fland=0.
    fland(4)=1.

    wetflag = .false.
    rainflag = .false.
    dewflag = .false.

    if (CMC .gt. 1.e-10) then
      if (DEW .gt. 1.e-10) then
         dewflag = .true.
      else
         rainflag = .true.
      endif
         wetflag = .true.
    endif

    call nonstomatal_zhang (sd, XLAI, SFCTMP, T1, RHf, fcld, prec, fland, USTAR, rainflag, dewflag,  &
                                Rb, Rac, Rg, Rcut, Rns, Dratio, Rm)
    
!print*,'nonstomatal_zhang done!'
    call CalCoszen( julday, ihour, latitude, COSZEN )
!print*,'coszen done'

    call DRYVD ( zlvl_wind, SFCSPD, sd,     &
                 SFCTMP, T1, SOLDN, RHf,    &
                 fcld, prec, COSZEN, pmb,   &
                 julday, fland, USTAR,      &
                 Ra_z, Rb, Rs_z, Rac, Rg, Rcut, Rns, Rc_z, Dratio, Rm, VDG, VDP ) 

!print*,'DRYVD done!'
    Wst=0.
    if (wetflag.and.SOLDN.GT.200.) then
      Wst=(SOLDN-200.)/800.
      Wst=amin1(Wst, 0.5)
      Wst=(SOLDN-200.)/800.*2.
      Wst=amin1(Wst, 0.8)
    end if
    if (wetflag) then
      Wst=0.8
    end if
!print*,'Wst=',Wst
    Rs_block=Rs/(1.-Wst)
!print*,'Rsblock=',Rs_Block
    do J=1, NG
       Rc(J) = 1./( (1.-Wst)/(Rs*Dratio(J)+Rm(J))+1./(Rac+Rg(J))+1./Rcut(J) )
       Vd(J) = 100./(Ra + Rb(J) + Rc(J))
!print*,'J=',J
    enddo
!print*,'dep_driver2 done'
!    print*,'Ra, RB,Rc,and Vd=',Ra, Rb(1), Rc(1), Vd(1),Dratio(10)
    
 END SUBROUTINE dep_driver2 

 SUBROUTINE AerodynRes(ZLM, Z0, CZIL, USTAR,RLMO,RA) 

      IMPLICIT NONE
      REAL     VKRM, EXCM 
      REAL     PIHF, ZTMIN, ZTMAX, SQVISC
      REAL     RIC, RRIC, FHNEU, RFC, RFAC, ZZ, PSLMU, PSLMS, PSLHU,     &
     & PSLHS
      REAL     XX, PSPMU, YY, PSPMS, PSPHU, PSPHS, ZLM, Z0
      REAL     CZIL, AKMS, AKHS, ZILFC, ZU, ZT, RDZ, CXCH
      REAL     DTHV, USTAR, ZSLU, ZSLT, RLOGU, RLOGT
      REAL     RLMO, ZETALT, ZETALU, ZETAU, ZETAT, XLU4, XLT4, XU4, XT4

      REAL     XLU, XLT, XU, XT, SIMM, PSHZ, SIMH, USTARK, RLMN,  &
     &         RLMA

      INTEGER  ITRMX, ILECH, ITR
      REAL SIMC, AKCS, RA

      PARAMETER                                                         &
     &        (VKRM = 0.40,EXCM = 0.001,ITRMX = 30,PIHF = 3.14159265/2.)      
      PARAMETER                                                         &
     &         (ZTMIN = -5.,ZTMAX = 1.,SQVISC = 258.2)
      PARAMETER                                                         &
     &       (RIC = 0.183,RRIC = 1.0/ RIC,FHNEU = 0.8,RFC = 0.191       &
     &        ,RFAC = RIC / (FHNEU * RFC * RFC))


! ----------------------------------------------------------------------
! NOTE: THE TWO CODE BLOCKS BELOW DEFINE FUNCTIONS
! ----------------------------------------------------------------------
! LECH'S SURFACE FUNCTIONS
! ----------------------------------------------------------------------
      PSLMU (ZZ)= -0.96* log (1.0-4.5* ZZ)
      PSLMS (ZZ)= ZZ * RRIC -2.076* (1. -1./ (ZZ +1.))
      PSLHU (ZZ)= -0.96* log (1.0-4.5* ZZ)
      PSLHS (ZZ)= ZZ * RFAC -2.076* (1. -1./ (ZZ +1.))

! ----------------------------------------------------------------------
! PAULSON'S SURFACE FUNCTIONS
! ----------------------------------------------------------------------
      PSPMU (XX)= -2.* log ( (XX +1.)*0.5) - log ( (XX * XX +1.)*0.5)   &
     &        +2.* ATAN (XX)                                            &
     &- PIHF
      PSPMS (YY)= 5.* YY
      PSPHU (XX)= -2.* log ( (XX * XX +1.)*0.5)
      PSPHS (YY)= 5.* YY


      ILECH = 0

      ZILFC = - CZIL * VKRM * SQVISC
      ZU = Z0

      RDZ = 1./ ZLM
      CXCH = EXCM * RDZ

      ZT = EXP (ZILFC * SQRT (USTAR * Z0))* Z0

      ZSLT = ZLM + ZT

      ZETALT = ZSLT * RLMO
!      ZETALT = MAX (ZSLT * RLMO,ZTMIN)
      ZETAT = ZT * RLMO

      IF (ILECH .eq. 0) THEN    
 
        IF (RLMO .lt. 0.)THEN

           XLT4 = 1. -16.* ZETALT
           XT4 = 1. -16.* ZETAT

           XLT = SQRT (SQRT (XLT4))
           XT = SQRT (SQRT (XT4))

           PSHZ = PSPHU (XT)
           SIMC = PSPHU (XLT) - PSHZ + log(ZLM/ZU)
        ELSE
          
!           ZETALT = MIN (ZETALT,ZTMAX)

           PSHZ = PSPHS (ZETAT)
           SIMC= PSPHS (ZETALT) - PSHZ + log(ZLM/ZU)
        END IF

      END IF

      USTARK = USTAR * VKRM
!      AKCS = MAX (USTARK / SIMC,CXCH)
      AKCS = USTARK / SIMC
      RA = 1./AKCS

 END SUBROUTINE AerodynRes

 SUBROUTINE CalCoszen(JDAY, IH, GLAT, COSZEN )
! -- Calculate solar zenith angle
   implicit none

   INTEGER :: JDAY, IH
   REAL :: GLAT, COSZEN
   REAL :: hour, DECLIN, SHORT1, SHORT2, COSZE

    hour=real(IH)
    DECLIN=ASIN(SIN(23.5*3.14159/180.)*      &
             SIN((JDAY-81.)*2.*3.14159/365.))
    SHORT1=SIN(GLAT)*SIN(DECLIN)
    SHORT2=COS(GLAT)*COS(DECLIN)
    COSZE=(HOUR-12)*3.14159/12.
    COSZEN=SHORT1+SHORT2*COS(COSZE)

 END SUBROUTINE CalCoszen

END MODULE module_noahlsm_dry_dep
