MODULE module_noahmplsm_dry_dep_rb

!use module_sf_noahmplsm
CONTAINS
!Subroutines that calculate RB:  
SUBROUTINE cal_rb(UU,VV,ZLVL,LAI,SAI,Q2,SFCPRS,SFCTMP,CM,TG,TAH,rb) ! Output: rb
  IMPLICIT NONE

! --------------------------------------------------------------------------------------------------
! inputs
!  INTEGER :: numgas
  real :: DLEAF, CWP, HCAN, Z0M
  real :: UR, UU,VV,UC,ZLVL
  real :: LAI,SAI
  REAL                  :: VAI    !total LAI + stem area index, one sided
  real :: MOZG, MOLG, ZPD, MPE, HG_1,HG,TG,TMP2,Z0H,Z0HG,Z0MG
  real :: QAIR, Q2, EAIR, SFCPRS, RHOAIR, SFCTMP
  REAL :: FV, CM
  REAL :: TMP1,TAH,FHGNEW,FHG,TMPRAH2,KH,RAHG
!  REAL                  :: HCAN   !canopy height (m) [note: hcan >= z0mg]
!  REAL                  :: UC     !wind speed at top of canopy (m/s)
!  REAL                  :: CWP    !canopy wind parameter
!  REAL                  :: FHG    !stability correction
  REAL, PARAMETER :: VKC    = 0.40      !von Karman constant
  REAL, PARAMETER :: GRAV   = 9.80616   !acceleration due to gravity (m/s2)
  REAL, PARAMETER :: CPAIR  = 1004.64   !heat capacity dry air at const pres (j/kg/k)
  REAL, PARAMETER :: RAIR   = 287.04    !gas constant for dry air (j/kg/k)
! outputs
  REAL                             :: rb     !bulk leaf boundary layer resistance (s/m)

  REAL :: TMPRB        !temporary calculation for rb
  real :: CWPC
!  real :: DLEAF
!  INTEGER :: l
!     CALCULATE:
!  DO l = 1, numgas
       DLEAF= 0.04
! --------------------------------------------------------------------------------------------------
       CWP = 0.18
       HCAN = 17.0
       Z0M = 1.0
       Z0H=Z0M
       UR = MAX( SQRT(UU**2.+VV**2.), 1. )
       UC = UR*LOG(HCAN/Z0M)/LOG(ZLVL/Z0M)
       VAI = LAI+ SAI

! stability correction to below canopy resistance

       MOZG = 0.
       MOLG = 0.
       ZPD = 0.65 * HCAN
       Z0MG = 0.01
       Z0HG=Z0MG
       MPE = 1.E-06       
       HG_1 = 0.
       QAIR   = Q2 / (1.0+Q2)       
       EAIR   = QAIR*SFCPRS / (0.622+0.378*QAIR)
       RHOAIR = (SFCPRS-0.378*EAIR) / (RAIR*SFCTMP)
       FV = UR * SQRT(CM)
      
!       IF(ITER > 1) THEN        
!        HG = RHOAIR*CPAIR*(TG  - TAH)   /RAHG
        TMP1 = VKC * (GRAV/TAH) * HG_1/(RHOAIR*CPAIR)
        IF (ABS(TMP1) .LE. MPE) TMP1 = MPE
        MOLG = -1. * FV**3 / TMP1
        MOZG = MIN( (ZPD-Z0MG)/MOLG, 1.)
!       END IF

       IF (MOZG < 0.) THEN
          FHGNEW  = (1. - 15.*MOZG)**(-0.25)
       ELSE
          FHGNEW  = 1.+ 4.7*MOZG
       ENDIF

!       IF (ITER == 1) THEN
          FHG = FHGNEW
!       ELSE
!          FHG = 0.5 * (FHG+FHGNEW)
!       ENDIF

       CWPC = (CWP * VAI * HCAN * FHG)**0.5

       TMP1 = EXP( -CWPC*Z0HG/HCAN )
       TMP2 = EXP( -CWPC*(Z0H+ZPD)/HCAN )
       TMPRAH2 = HCAN*EXP(CWPC) / CWPC * (TMP1-TMP2)

! aerodynamic resistances raw and rah between heights zpd+z0h and z0hg.

       KH  = MAX ( VKC*FV*(HCAN-ZPD), MPE )
       RAHG = TMPRAH2 / KH
       HG = RHOAIR*CPAIR*(TG  - TAH)   /RAHG
       TMP1 = VKC * (GRAV/TAH) * HG/(RHOAIR*CPAIR)
        IF (ABS(TMP1) .LE. MPE) TMP1 = MPE
        MOLG = -1. * FV**3 / TMP1
        MOZG = MIN( (ZPD-Z0MG)/MOLG, 1.)

       IF (MOZG < 0.) THEN
          FHGNEW  = (1. - 15.*MOZG)**(-0.25)
       ELSE
          FHGNEW  = 1.+ 4.7*MOZG
       ENDIF

!       IF (ITER == 1) THEN
!          FHG = FHGNEW
!       ELSE
          FHG = 0.5 * (FHG+FHGNEW)
!       ENDIF

       CWPC = (CWP * VAI * HCAN * FHG)**0.5
! leaf boundary layer resistance

       TMPRB  = CWPC*50. / (1. - EXP(-CWPC/2.))
       rb     = TMPRB * SQRT(DLEAF/UC)
!  END DO
END SUBROUTINE cal_rb

END MODULE module_noahmplsm_dry_dep_rb


