MODULE module_noahmplsm_dry_dep_rc

use module_sf_noahmplsm, only: RSMIN,RSMAX,HS,RGL,TOPT
!use module_sf_noahlsm, only: RCSOIL

CONTAINS

!Subroutines that calculate RC:  

!        IF (OPT_CRS == 1) then  ! Ball-Berry
!         CALL STOMATA (VEGTYP,MPE   ,PARSUN ,FOLN  ,ILOC  , JLOC , & !in       
!                       TV    ,ESTV  ,EAH    ,SFCTMP,SFCPRS, & !in
!                       O2AIR ,CO2AIR,IGS    ,BTRAN ,RB    , & !in
!                       RSSUN ,PSNSUN)
!        IF (OPT_CRS == 2) then  ! Jarvis
!         CALL  CANRES (PARSUN,TV    ,BTRAN ,EAH    ,SFCPRS, & !in
!                       RSSUN ,PSNSUN,ILOC  ,JLOC   )          !out
SUBROUTINE cal_rc(PAR, SFCTMP,SMC,SMCWLT,SMCREF,ZSOIL,NROOT,NSOIL, EAH   ,SFCPRS,         & ! Output: rc
                  RCQ, RCS,   RCT, RCSOIL ,RC)                        
                       
    IMPLICIT NONE
! --------------------------------------------------------------------------------------------------
! inputs

    REAL                       :: PAR    !par absorbed per unit sunlit lai (w/m2)
    REAL                       :: SFCTMP !canopy air temperature
    REAL                       :: SFCPRS !surface pressure (pa)
    REAL                       :: EAH    !water vapor pressure (pa)
    REAL                       :: RCSOIL !soil moisture stress factor
!    REAL :: RSMIN,RSMAX,RGL,
!outputs,add the out of RCQ,RCS,RCT.

    REAL                      :: RC     !canopy resistance per unit LAI
    REAL                      :: PSN    !foliage photosynthesis (umolco2/m2/s)
    REAL                      :: RCQ    !edit by chang
    REAL                      :: RCS    !edit by chang
    REAL                      :: RCT    !edit by chang
!local

    REAL                                  :: FF
    REAL                                  :: Q2     !water vapor mixing ratio (kg/kg)
    REAL                                  :: Q2SAT  !saturation Q2
    REAL                                  :: DQSDT2 !d(Q2SAT)/d(T)
      INTEGER :: NROOT,NSOIL
      INTEGER :: K
      REAL,DIMENSION(1:NSOIL) :: SMC,ZSOIL
      REAL                :: GX,SMCWLT,SMCREF  !,SMC,ZSOIL,PART
      REAL,DIMENSION(1:NSOIL) ::  PART
! RSMIN, RSMAX, TOPT, RGL, HS are canopy stress parameters set in REDPRM
! ----------------------------------------------------------------------
! initialize canopy resistance multiplier terms.
! ----------------------------------------------------------------------
    RC     = 0.0
    RCS    = 0.0
    RCT    = 0.0
    RCQ    = 0.0
!    RCSOIL = 0.0
!  compute Q2 and Q2SAT

    Q2 = 0.622 *  EAH  / (SFCPRS - 0.378 * EAH) !specific humidity [kg/kg]
    Q2 = Q2 / (1.0 + Q2)                        !mixing ratio [kg/kg]

    CALL CALHUM(SFCTMP, SFCPRS, Q2SAT, DQSDT2)
!print*,'Q2SAT=',Q2SAT
! contribution due to incoming solar radiation

    FF  = 2.0 * PAR / RGL
!print*,'FF=',FF
    RCS = (FF + RSMIN / RSMAX) / (1.0+ FF)
!print*,'RCS=',RCS
    RCS = MAX (RCS,0.0001)

! contribution due to air temperature

    RCT = 1.0- 0.0016* ( (TOPT - SFCTMP)**2.0)
    RCT = MAX (RCT,0.0001)
!print*,'RCT=',RCT
! contribution due to vapor pressure deficit

    RCQ = 1.0/ (1.0+ HS * MAX(0.,Q2SAT-Q2))
    RCQ = MAX (RCQ,0.01)
!print*,'RCQ=',RCQ
! ----------------------------------------------------------------------
! USE SOIL DEPTH AS WEIGHTING FACTOR
! ----------------------------------------------------------------------
! ----------------------------------------------------------------------
! USE ROOT DISTRIBUTION AS WEIGHTING FACTOR
!      PART(1) = RTDIS(1) * GX
!    BTRAN = 0.
!
!     IF(IST ==1 ) THEN
!       DO IZ = 1, NROOT
!          IF(OPT_BTR == 1) then                  ! Noah
!            GX    = (SH2O(IZ)-SMCWLT) / (SMCREF-SMCWLT)
!          END IF
!          IF(OPT_BTR == 2) then                  ! CLM
!            PSI   = MAX(PSIWLT,-PSISAT*(MAX(0.01,SH2O(IZ))/SMCMAX)**(-BEXP) )
!            GX    = (1.-PSI/PSIWLT)/(1.+PSISAT/PSIWLT)
!          END IF
!          IF(OPT_BTR == 3) then                  ! SSiB
!            PSI   = MAX(PSIWLT,-PSISAT*(MAX(0.01,SH2O(IZ))/SMCMAX)**(-BEXP) )
!            GX    = 1.-EXP(-5.8*(LOG(PSIWLT/PSI)))
!          END IF
!
!          GX = MIN(1.,MAX(0.,GX))
!          BTRANI(IZ) = MAX(MPE,DZSNSO(IZ) / (-ZSOIL(NROOT)) * GX)
!          BTRAN      = BTRAN + BTRANI(IZ)
!       END DO
!       BTRAN = MAX(MPE,BTRAN)
!
!       BTRANI(1:NROOT) = BTRANI(1:NROOT)/BTRAN
!     END IF

! ----------------------------------------------------------------------
!      GX = (SMC (1) - SMCWLT) / (SMCREF - SMCWLT)
!print*,'GX=',GX
!      IF (GX  >  1.) GX = 1.
!      IF (GX  <  0.) GX = 0.
!print*,'GX=',GX
!      PART(1) = (ZSOIL(1)/ ZSOIL(NROOT)) * GX
!print*,'PART=',PART
!      DO K = 2,NROOT
!         GX = (SMC (K) - SMCWLT) / (SMCREF - SMCWLT)
!         IF (GX >  1.) GX = 1.
!         IF (GX <  0.) GX = 0.
! ----------------------------------------------------------------------
! USE SOIL DEPTH AS WEIGHTING FACTOR
! ----------------------------------------------------------------------
! ----------------------------------------------------------------------
! USE ROOT DISTRIBUTION AS WEIGHTING FACTOR
!        PART(K) = RTDIS(K) * GX
! ----------------------------------------------------------------------
!         PART (K) = ( (ZSOIL (K) - ZSOIL (K -1))/ ZSOIL (NROOT)) * GX
!      END DO
!      DO K = 1,NROOT
!         RCSOIL = RCSOIL + PART (K)
!      END DO
!print*,'RCSOIL=',RCSOIL
! ----------------------------------------------------------------------
! DETERMINE CANOPY RESISTANCE DUE TO ALL FACTORS.  CONVERT CANOPY
! RESISTANCE (RC) TO PLANT COEFFICIENT (PC) TO BE USED WITH POTENTIAL
! EVAP IN DETERMINING ACTUAL EVAP.  PC IS DETERMINED BY:
!   PC * LINERIZED PENMAN POTENTIAL EVAP =
!   PENMAN-MONTEITH ACTUAL EVAPORATION (CONTAINING RC TERM).
! ----------------------------------------------------------------------
!      RCSOIL = MAX (RCSOIL,0.0001)
! determine canopy resistance due to all factors
!print*,'RCSOIL=',RCSOIL
    RC  = RSMIN / (RCS * RCT * RCQ * RCSOIL)
    PSN = -999.99       ! PSN not applied for dynamic carbon
!print*,'RC=',RC
END SUBROUTINE cal_rc

!----------------------------------------------------------------------------------------
        SUBROUTINE CALHUM(SFCTMP, SFCPRS, Q2SAT, DQSDT2)

        IMPLICIT NONE

        REAL, INTENT(IN)       :: SFCTMP, SFCPRS
        REAL, INTENT(OUT)      :: Q2SAT, DQSDT2
        REAL, PARAMETER        :: A2=17.67,A3=273.15,A4=29.65, ELWV=2.501E6,         &
                                  A23M4=A2*(A3-A4), E0=0.611, RV=461.0,             &
                                  EPSILON=0.622
        REAL                   :: ES, SFCPRSX

! Q2SAT: saturated mixing ratio
        ES = E0 * EXP ( ELWV/RV*(1./A3 - 1./SFCTMP) )
! convert SFCPRS from Pa to KPa
        SFCPRSX = SFCPRS*1.E-3
        Q2SAT = EPSILON * ES / (SFCPRSX-ES)
! convert from  g/g to g/kg
        Q2SAT = Q2SAT * 1.E3
! Q2SAT is currently a 'mixing ratio'

! DQSDT2 is calculated assuming Q2SAT is a specific humidity
        DQSDT2=(Q2SAT/(1+Q2SAT))*A23M4/(SFCTMP-A4)**2

! DG Q2SAT needs to be in g/g when returned for SFLX
        Q2SAT = Q2SAT / 1.E3

        END SUBROUTINE CALHUM
!---------------------------------------------------------------------------------------
END MODULE module_noahmplsm_dry_dep_rc


