    MODULE module_wddm

    CONTAINS

 SUBROUTINE wddm_driver(iyear,julday,ivegtyp,numgas,z,d,rdtheta,z0,                    & ! I
                          ta,rad,RHf,prcp,ustar,rmol,                                    & ! I       
                          vd, ra,rb, rc, rsx,rmx,rns,rlux,rclx,rgsx,rdc,rac,resice)            ! O

   IMPLICIT NONE
   INTEGER :: iyear,julday, numgas, ivegtyp
   REAL :: z, d, rdtheta
   REAL :: ta,rad,RHf,prcp
   REAL :: ustar,z0,rmol 
   real, parameter :: karman=0.40 

! resistance & deposition velocities
  REAL, DIMENSION(numgas) :: rb, rns,rc, vd, rsx,rmx,rlux,rclx,rgsx
  REAL :: ra, rdc,rac,resice

! .. Local Scalars ..
  REAL, DIMENSION(numgas) :: hstar, f0, Dratio,dhr, sc
  INTEGER :: n, LB, iseason, iland, p_so2, p_o3
  LOGICAL :: dewflag, rainflag
  REAL :: rh   
  REAL :: Pr
  INTEGER :: mapusgs(25)
  DATA mapusgs/1, 2, 2, 2, 2, 4, 3, 3, 3, 3, 4, 5, 4, 5, 6, 7, 9, 6, 8, 9, 6, 6, 8, 0, 0/ 
                                                  
                                                     
! .. Intrinsic Functions ..                       
      INTRINSIC max, min
                             
!-- Determin season and landuse type in Wesely's category
!     compute leapyear bias (1 or 0), and season.

      IF(MOD(iyear,4).EQ.0) THEN
        LB=1
      ELSE
        LB=0
      END IF

      IF(julday.GE.(152+LB).AND.julday.LT.(244+LB)) THEN   
        iseason = 1        ! Midsummer with lush vegetation: Jun - Aug
      ELSE IF(julday.GE.(244+LB).AND.julday.LT.(305+LB)) THEN
        iseason = 2        ! Autumn with unharvested cropland: Sep - Oct
      ELSE IF(julday.GE.(305+LB).AND.julday.LT.(335+LB)) THEN
        iseason = 3        ! late Autumn after frost but no snow: Nov 
      ELSE IF(julday.GE.(335+LB).OR.julday.LT.(60+LB)) THEN
        iseason = 4        ! Winter with snow on the ground: Dec - Feb
      ELSE IF(julday.GE.(60+LB).AND.julday.LT.(152+LB)) THEN
        iseason = 5        ! transitional Spring with partial green coverage: Mar - May
      END IF                                            

!      iseason = 1
!      if(julday.lt.90.or.julday.gt.270)then
!         iseason=2
!      endif
  
      iland = mapusgs(ivegtyp)

!      print*,'iseason,iland=',iseason,iland
 
!-- Determin rainflag and dewflag

!     Set logical default values
      dewflag = .FALSE.
      rainflag = .FALSE.

!      rh = MIN(100., 100.*qv / (3.80*exp(17.27*(ta-273.)/(ta-36.))/(sfcprs/100.)))
!      rh = MAX(5.,rh)
      if (RHf >= 0.95) dewflag = .true.

      if(prcp.gt.0.) rainflag = .true.
      
!--- deposition

      CALL dep_init(numgas,hstar,f0,Dratio,dhr,sc,p_so2,p_o3)

      CALL ra_McRae(karman,rmol,z,d,z0,ustar,                         & ! I
                    ra)                                                 ! O
  
      Pr=0.72
      CALL calrb(numgas,sc,pr,ustar,karman,                           & ! I
                 rb)                                                    ! O

      CALL rc_wesely(ta,rad,rdtheta,hstar,f0,Dratio,dhr,iland,iseason,dewflag,rainflag,numgas,p_so2,p_o3,       & ! I
                     rsx,rmx,rlux,rclx,rgsx,rdc,rac,resice,rns,rc)                                                    ! O

      DO n = 1, numgas
        vd(n) = 1./(ra+rb(n)+rc(n))
      END DO
      
      RETURN

   END SUBROUTINE wddm_driver

   SUBROUTINE dep_init(numgas,hstar,f0,Dratio,dhr,sc,p_so2,p_o3)

      IMPLICIT NONE
      INTEGER :: numgas

      INTEGER :: p_so2, p_so4, p_no2, p_no, p_o3, p_hno3, p_h2o2,       &
                 p_ald, p_hcho, p_op, p_paa, p_ora, p_nh3, p_pan,       &
                 p_hono, p_co
      

      REAL, DIMENSION(numgas) :: hstar, f0, Dratio 

      REAL :: dvj(numgas),dhr(numgas)
      REAL :: sc(numgas)

! .. Local variable .. 
      INTEGER :: l

      p_so2 = 1
      p_o3  = 2
      p_no2 = 3
      p_no  = 4
      p_hno3= 5
      p_h2o2= 6
      p_ald = 7
      p_hcho= 8
      p_op  = 9
      p_paa =10
      p_ora =11
      p_nh3 =12
      p_pan =13
      p_hono=14
      p_co  =15

!     HENRY''S LAW COEFFICIENTS
!     Effective Henry''s law coefficient at pH 7
!     [KH298]=mole/(l atm)
 
      hstar(p_so2) = 2.53E+5
      hstar(p_o3)   = 1.13E-2
      hstar(p_no2) = 6.40E-3
      hstar(p_no) = 1.90E-3
      hstar(p_hno3) = 2.69E+13
      hstar(p_h2o2) = 7.45E+4
      hstar(p_ald) = 1.14E+1
      hstar(p_hcho) = 2.97E+3
      hstar(p_op) = 2.21E+2
      hstar(p_paa) = 4.73E+2
      hstar(p_ora) = 9.63E+5
      hstar(p_nh3) = 1.04E+4
      hstar(p_pan) = 2.97E+0
      hstar(p_hono) = 3.47E+5
      hstar(p_co) = 8.20E-3

!     REACTIVITY FACTORS

      f0(p_so2) = 0.
      f0(p_o3)   = 1.
      f0(p_no2) = 0.1
      f0(p_no) = 0.
      f0(p_hno3) = 0.
      f0(p_h2o2) = 1.
      f0(p_ald) = 0.
      f0(p_hcho) = 0.
      f0(p_op) = 0.1
      f0(p_paa) = 0.1
      f0(p_ora) = 0.
      f0(p_nh3) = 0.
      f0(p_pan) = 0.1
      f0(p_hono) = 0.1
      f0(p_co) = 0.

!      Dratio(p_so2) = 1.9
!      Dratio(p_o3)   = 1.6
!      Dratio(p_no2) = 1.6
!      Dratio(p_no) = 1.3
!      Dratio(p_hno3) = 1.9
!      Dratio(p_h2o2) = 1.4
!      Dratio(p_ald) = 1.6
!      Dratio(p_hcho) = 1.3
!      Dratio(p_op) = 1.6
!      Dratio(p_paa) = 2.
!      Dratio(p_ora) = 1.6
!      Dratio(p_nh3) = 1.0
!      Dratio(p_pan) = 2.6
!      Dratio(p_hono) = 1.6
!      Dratio(p_co) = 1.2

!     DIFFUSION COEFFICIENTS/MOLECULAR DIFFUSIVITY
!     [DV]=cm2/s (assumed: 1/SQRT(molar mass) when not known)
      
      dvj(p_so2) = 0.126
      dvj(p_o3)   = 0.175
      dvj(p_no2) = 0.147
      dvj(p_no) = 0.183
      dvj(p_hno3) = 0.126
      dvj(p_h2o2) = 0.171
      dvj(p_ald) = 0.151
      dvj(p_hcho) = 0.183
      dvj(p_op) = 0.144
      dvj(p_paa) = 0.115
      dvj(p_ora) = 0.115
      dvj(p_nh3) = 0.227
      dvj(p_pan) = 0.091
      dvj(p_hono) = 0.153
      dvj(p_co) = 0.189

!     -DH/R (for temperature correction)
!     [-DH/R]=K
      
      dhr(p_so2) = 5816.  
      dhr(p_o3) = 2300.
      dhr(p_no2) = 2500.
      dhr(p_no) = 1480.
      dhr(p_hno3) = 8684.
      dhr(p_h2o2) = 6615.
      dhr(p_ald) = 6266.
      dhr(p_hcho) = 7190.
      dhr(p_op) = 5607.
      dhr(p_paa) = 6170.
      dhr(p_ora) = 5716.
      dhr(p_nh3) = 3660.
      dhr(p_pan) = 5760.
      dhr(p_hono) = 3775.
      dhr(p_co) = 0.

      DO l = 1, numgas
        dvj(l) = dvj(l)*(293.15/298.15)**1.75   ! Correction of diff. coeff
        sc(l) = 0.15/dvj(l)        ! 0.15 is the kenematic viscosity of air, Schmidt Number at 20C
!        dratio(l) = 0.242/dvj(l)   ! of water vapor and gas at
        dratio(l) = 0.227/dvj(l)
!--   Ratio of diffusion coeffi
!          scpr23(l) = (sc/0.72)**(2./3.) ! (Schmidt # / Prandtl #)**
      END DO

      RETURN   
   END SUBROUTINE dep_init

   SUBROUTINE RA_DDIM(u, ustar, z, d, RMOL, ra) ! Output: ra

! reference
! Horii et al. atmospheric reactive nitrogen concentration and flux budgets at a Northteastern U.S. forest site. 2005 
    IMPLICIT NONE
    real :: u, ustar, z, d, RMOL, ra
    real :: PsiH
    real, parameter :: karman=0.40
               
    if(RMOL.lt.0) then
      PsiH=exp(0.598+0.39*log(-1*(z-d)*RMOL)-0.09*(log(-1*(z-d)*RMOL))**2)
    else
      PsiH=-5*(z-d)*RMOL
    endif
        
    ra=u/(ustar**2.0)-PsiH/(karman*ustar)
        
    return

 END SUBROUTINE RA_DDIM 

 SUBROUTINE CALRA(z0, ustar, z, d, RMOL, ra) ! Output: ra

! reference:
! Wesely et al. some factors that affect the deposition rates of sulfur dioxide and similar gases on vegetation, AE. 1977
 
    IMPLICIT NONE
    real :: z0, ustar, z, d, RMOL, ra
    real :: PsiH
    real, parameter :: karman=0.40

    if(RMOL.lt.0) then
      PsiH=exp(0.598+0.39*log(-1*(z-d)*RMOL)-0.09*(log(-1*(z-d)*RMOL))**2)
    else
      PsiH=-5*(z-d)*RMOL
    endif

    ra=(log((z-d)/z0)-PsiH)/(karman*ustar)

    return

 END SUBROUTINE CALRA

 SUBROUTINE ra_McRae(karman,rmol,z,d,z0,ustar,                             & ! I
                          ra)                                                  ! O

      IMPLICIT NONE
      REAL :: karman,rmol,z,d,z0,ustar
      REAL :: ra
! .. Local variable ..
      REAL :: ar,ao,polint

!     RMOL     - RECIPROCAL OF THE MONIN-OBUKHOV LENGTH (1/L)
!     Z        - REFERENCE HEIGHT ABOVE GROUND
!     D        - DISPLACEMENT HEIGHT
!     Z0       - SURFACE ROUGHNESS HEIGHT
!     KARMAN   - VON KARMAN CONSTANT (k)
!     USTAR    - FRICTION VELOCITY (U*)
!     POLINT   - POLLUTANT INTEGRAL
!     RA       - AERODYNAMIC RESISTANCE

!     DETERMINE THE STABILITY BASED ON THE CONDITIONS
!             1/L < 0 --> UNSTABLE
!             1/L = 0 --> NEUTRAL
!             1/L > 0 --> STABLE

        if(abs(rmol) < 1.E-6 ) rmol = 0.

        IF (rmol<0) THEN
          ar = ((1.0-9.0*(z-d)*rmol)**(0.25)+0.001)**2
          ao = ((1.0-9.0*z0*rmol)**(0.25)+0.001)**2
          polint = 0.74*(alog((ar-1.0)/(ar+1.0))-alog((ao-1.0)/(ao+1.0)))
        ELSE IF (rmol==0.) THEN
          polint = 0.74*alog((z-d)/z0)
        ELSE
          polint = 0.74*alog((z-d)/z0) + 4.7*rmol*((z-d)-z0)
        END IF

!     CALCULATE THE AERODYNAMIC RESISTANCE(ra)

        ra=polint/(karman*max(ustar,1.0e-4))
         
        RETURN
 END SUBROUTINE ra_McRae

! **************************  SUBROUTINE calrb  ***************************
 SUBROUTINE calrb(numgas,sc,pr,ustar,karman,                    & ! I
                  rb)                                             ! O

!     SCPR23   - (Schmidt #/Prandtl #)**(2/3) Diffusion correction fact
!     KARMAN   - VON KARMAN CONSTANT
!     USTAR    - FRICTION VELOCITY U*

      IMPLICIT NONE 
      INTEGER :: numgas
      REAL :: ustar, karman, pr
      REAL :: sc(numgas)
      REAL :: rb(numgas)

! .. Local variable ..      
      INTEGER :: l

        DO l = 1, numgas
          rb(l)=2.0*(sc(l)/pr)**(2./3.) /(ustar*karman)
        END DO

        RETURN
 END SUBROUTINE calrb
      
! **************************  SUBROUTINE RC  ***************************
 SUBROUTINE rc_wesely(t,rad,rdtheta,hstar,f0,Dratio,dhr,iland,iseason,dewflag,rainflag,numgas,p_so2,p_o3,         & ! I
                      rsx,rmx,rlux,rclx,rgsx,rdc,rac2,resice,rns,rcx)                                               ! O

!     THIS SUBROUTINE CALCULATES SURFACE RESISTENCES ACCORDING
!     TO THE MODEL OF M. L. WESELY, ATMOSPHERIC ENVIRONMENT 23 (1989), 1293-1304
!     WITH SOME ADDITIONS ACCORDING TO J. W. ERISMAN, A. VAN PUL, AND P. WYERS,
!     ATMOSPHERIC ENVIRONMENT 28 (1994), 2595-2607
!     WRITTEN BY  WINFRIED SEIDL, APRIL 1997
!     MODYFIED BY WINFRIED SEIDL, MARCH 2000
!                    FOR MM5 VERSION 3

        IMPLICIT NONE
        
        REAL, INTENT(IN) :: rad, t, rdtheta
        INTEGER, INTENT(IN) :: numgas, iland, iseason, p_so2, p_o3
        LOGICAL, INTENT(IN) :: rainflag, dewflag
        REAL,DIMENSION(numgas), INTENT(IN) :: hstar,f0,Dratio,dhr
        
        REAL,DIMENSION (numgas), INTENT(OUT) :: rsx, rmx, rlux, rclx, rgsx, rns, rcx
        REAL, INTENT(OUT) :: rdc,  resice, rac2

! .. Local variable ..
        INTEGER, PARAMETER :: numseason=5, numlu=11
        REAL,DIMENSION(numgas) :: hstary
        REAL, DIMENSION (numseason,numlu) :: Ri, Rlu,Rac,Rgss,Rgso,Rcls,Rclo
        INTEGER :: n, l
        REAL :: tc, z, rluo1, rluo2, rs

! #1, Midsummer with lush vegetation
        DATA (RI(1,L),L=1,numlu)   /1.E25, 60., 120., 70., 130.,              &
                                    100., 1.E25, 1.E25, 80., 100., 150./
        DATA (RLU(1,L),L=1,numlu)  /1.E25, 2000., 2000., 2000., 2000.,        &
                                    2000., 1.E25, 1.E25, 2500., 2000., 4000./
        DATA (RAC(1,L),L=1,numlu)  /100., 200., 100., 2000., 2000.,          &
                                    2000., 0.1, 0.1, 300., 150., 200./
        DATA (RGSS(1,L),L=1,numlu) /400., 150., 350., 500., 500.,            &
                                    100., 0.1, 1000., 0.1, 220., 400./
        DATA (RGSO(1,L),L=1,numlu) /300., 150., 200., 200., 200.,            &
                                    300., 2000., 400., 1000., 180., 200./
        DATA (RCLS(1,L),L=1,numlu) /1.E25, 2000., 2000., 2000., 2000.,        &
                                    2000., 1.E25, 1.E25, 2500., 2000., 4000./
        DATA (RCLO(1,L),L=1,numlu) /1.E25, 1000., 1000., 1000., 1000.,        &
                                    1000., 1.E25, 1.E25, 1000., 1000., 1000./
! #2, Autumn with unharvested cropland
        DATA (RI(2,L),L=1,numlu)   /1.E25, 1.E25, 1.E25, 1.E25, 250.,            &
                                    500., 1.E25, 1.E25, 1.E25, 1.E25, 1.E25/
        DATA (RLU(2,L),L=1,numlu)  /1.E25, 9000., 9000., 9000., 4000.,        &
                                    8000., 1.E25, 1.E25,  9000.,  9000., 9000./
        DATA (RAC(2,L),L=1,numlu)  /100., 150., 100., 1500., 2000.,          &
                                    1700., 0.1, 0.1, 200., 120., 140./
        DATA (RGSS(2,L),L=1,numlu) /400., 200., 350., 500., 500.,            &
                                    100., 0.1, 1000., 0.1, 300., 400./
        DATA (RGSO(2,L),L=1,numlu) /300., 150., 200., 200., 200.,            &
                                    300., 2000., 400., 800., 180., 200./
        DATA (RCLS(2,L),L=1,numlu) /1.E25, 9000., 9000., 9000., 2000.,        &
                                    4000., 1.E25, 1.E25, 9000., 9000., 9000./
        DATA (RCLO(2,L),L=1,numlu) /1.E25, 400., 400., 400., 1000.,           &
                                    600., 1.E25, 1.E25 , 400., 400., 400./
! #3, Late autumn after frost, no snow
        DATA (RI(3,L),L=1,numlu)   /1.E25, 1.E25, 1.E25, 1.E25, 250.,            &
                                    500., 1.E25, 1.E25, 1.E25, 1.E25, 1.E25/
        DATA (RLU(3,L),L=1,numlu)  /1.E25, 1.E25, 9000., 9000., 4000.,         &
                                    8000., 1.E25, 1.E25, 9000., 9000,  9000./
        DATA (RAC(3,L),L=1,numlu)  /100., 10., 100., 1000., 2000.,           &
                                    1500., 0.1, 0.1, 100., 50., 120./
        DATA (RGSS(3,L),L=1,numlu) /400., 150., 350., 500., 500.,            &        
                                    200., 0.1, 1000., 0.1, 200., 400./
        DATA (RGSO(3,L),L=1,numlu) /300., 150., 200., 200., 200.,            &
                                    300., 2000., 400., 1000., 180., 200./
        DATA (RCLS(3,L),L=1,numlu) /1.E25, 1.E25, 9000., 9000., 3000.,         &
                                    6000., 1.E25, 1.E25, 9000., 9000., 9000./
        DATA (RCLO(3,L),L=1,numlu) /1.E25, 1000., 400., 400., 1000.,          &
                                    600., 1.E25, 1.E25, 800., 600., 600./
! #4, Winter with snow on the ground
        DATA (RI(4,L),L=1,numlu)   /1.E25, 1.E25, 1.E25, 1.E25, 400.,            &
                                    800., 1.E25, 1.E25, 1.E25, 1.E25, 1.E25/
        DATA (RLU(4,L),L=1,numlu)  /1.E25, 1.E25, 1.E25, 1.E25, 6000.,           &
                                    9000., 1.E25, 1.E25, 9000., 9000., 9000./
        DATA (RAC(4,L),L=1,numlu)  /100., 10., 10., 1000., 2000.,            &
                                    1500., 0.1, 0.1, 50., 10., 50./
        DATA (RGSS(4,L),L=1,numlu) /100., 100., 100., 100., 100.,            &
                                    100., 0.1, 1000., 100., 100., 50./
        DATA (RGSO(4,L),L=1,numlu) /600., 3500., 3500., 3500., 3500.,        &
                                    3500., 2000., 400., 3500., 3500., 3500./
        DATA (RCLS(4,L),L=1,numlu) /1.E25, 1.E25, 1.E25, 9000., 200.,           &
                                    400., 1.E25, 1.E25, 9000., 1.E25, 9000./
        DATA (RCLO(4,L),L=1,numlu) /1.E25, 1000., 1000., 400., 1500.,         &
                                    600., 1.E25, 1.E25, 800., 1000., 800./
! #5, Transitional spring with partial green coverage
        DATA (RI(5,L),L=1,numlu)   /1.E25, 120., 240., 140., 250.,            &
                                    190., 1.E25, 1.E25 , 160., 200., 300./
        DATA (RLU(5,L),L=1,numlu)  /1.E25, 4000., 4000., 4000., 2000.,        &
                                    3000., 1.E25, 1.E25, 4000., 4000., 8000./
        DATA (RAC(5,L),L=1,numlu)  /100., 50., 80., 1200., 2000.,            &
                                    1500., 0.1, 0.1, 200., 60., 120./
        DATA (RGSS(5,L),L=1,numlu) /500., 150., 350., 500., 500.,            &
                                    200., 0.1, 1000., 0.1, 250., 400./
        DATA (RGSO(5,L),L=1,numlu) /300., 150., 200., 200., 200.,            &
                                    300., 2000., 400., 1000., 180., 200./
        DATA (RCLS(5,L),L=1,numlu) /1.E25, 4000., 4000., 4000., 2000.,        &
                                    3000., 1.E25, 1.E25, 4000., 4000., 8000./
        DATA (RCLO(5,L),L=1,numlu) /1.E25, 1000., 500., 500., 1500.,          &
                                    700., 1.E25, 1.E25, 600., 800., 800./

! .. Intrinsic Functions ..
        INTRINSIC exp

        rcx = 1.
        tc = t - 273.15

        z = 200./(rad+0.1)

        IF ((tc<=0.) .OR. (tc>=40.)) THEN
          rs = 9999.
        ELSE
          rs = ri(iseason,iland)*(1+z*z)*(400./(tc*(40.-tc)))
        END IF

        resice = 1000.*exp(-tc-4.)
        rluo1 = 1./(1./3000.+1./3./rlu(iseason,iland))
        rluo2 = 1./(1./1000.+1./3./rlu(iseason,iland))
        rdc = 100*(1.+1000./(rad+10))/(1+1000.*rdtheta)
        rac2 = rac(iseason,iland)
    
        DO n = 1, numgas
!--Rsx
           rsx(n)=rs*Dratio(n)

           hstary(n) = hstar(n)*exp(dhr(n)*(1./t-1./298.))
!--Rmx
           rmx(n) = 1./(hstary(n)/3000.+100.*f0(n))
           if (n == p_so2) rmx(n) = 0.
           if (n == p_o3)  rmx(n) = 0.

!--Rlux
           rlux(n) = rlu(iseason,iland)/(1.E-5*hstary(n)+f0(n)) + resice

           IF (dewflag) THEN
             rlux(n) = 1./(1./3./rlux(n)+1.E-7*hstary(n)+f0(n)/rluo1)
             if (n == p_so2) then
                rlux(n) = 100
                if (iland == 1) rlux(n) = 50
             endif
             if (n == p_o3) rlux(n) = rluo1
           END IF

           IF (rainflag) THEN
             rlux(n) = 1./(1./3./rlux(n)+1.E-7*hstary(n)+f0(n)/rluo2)
             if (n == p_so2) then
                rlux(n) = 1./(1./5000.+1./3./rlu(iseason,iland))
                if (iland == 1) rlux(n) = 50
             endif
             if (n == p_o3) rlux(n) = rluo2          
           END IF

!--Rclx            
          rclx(n) = 1./(hstary(n)/1.E+5/rcls(iseason,iland)+f0(n)/rclo(iseason,iland)) + resice
          if (n == p_so2) rclx(n) = rcls(iseason,iland)
          if (n == p_o3)  rclx(n) = rclo(iseason,iland)
!--Rgsx
          rgsx(n) = 1./(hstary(n)/1.E+5/rgss(iseason,iland)+f0(n)/rgso(iseason,iland)) + resice
          if (n == p_so2) rgsx(n) = rgss(iseason,iland)
          if (n == p_o3)  rgsx(n) = rgso(iseason,iland)
!--Rns
          rns(n) = 1./(1./rlux(n)+1./(rdc+rclx(n))+1./(rac2+rgsx(n)))

!-----Rcx
          rcx(n) = 1./(1./(rsx(n)+rmx(n))+1./rns(n))
          IF (rcx(n)<1.) rcx(n) = 1.          

        ENDDO

        RETURN
      END SUBROUTINE rc_wesely

!.........................................................................
! Version "@(#)$Header$"
! EDSS/Models-3 I/O API.  Copyright (C) 1992-2002 MCNC
! Distributed under the GNU LESSER GENERAL PUBLIC LICENSE version 2.1
! See file "LGPL.txt" for conditions of use.
!.........................................................................

!      INTEGER FUNCTION JULIAN (YEAR, MNTH, MDAY)

!***********************************************************************
!  function body starts at line  66
!
!  FUNCTION:  returns the Julian day (1...365,366) corresponding to
!      the date MNTH-MDAY-YEAR.
!      NOTE:  This is NOT the Julian DATE -- only the
!      day-number.  To get the Julian date:
!
!      JDATE = 1000 * YEAR  +  JULIAN ( YEAR , MNTH , MDAY )
!
!  ARGUMENT LIST DESCRIPTION:
!
!    Input arguments:
!
!      YEAR     Calendar year
!      MNTH     Month of year  1, 12
!      MDAY     Day of month   1, 31
!
!     Output arguments:  none
!
!  RETURN VALUE:
!
!      JULIAN   The Julian DAY of the input arguments combined
!
!  REVISION HISTORY:
!
!    5/1988   Modified for ROMNET
!    8/1990   Modified for ROM 2.2 by Carlie J. Coats, Jr., CSC
!       improved comments; improved Zeller's Congruence algorithm
!       and using IF-THEN ... ELSE IF ... construction.
!
!    8/1999   Version for global-climate IO_360, which uses 360-day "year"
!       
!       2/2002 Unification by CJC with global-climate JULIAN
!***********************************************************************
!       Jan 2010 cancel IO_360 by zhiyong
!      IMPLICIT NONE

!...........   ARGUMENTS and their descriptions:

!        INTEGER   YEAR            ! year YYYY
!        INTEGER   MNTH            ! month 1...12
!        INTEGER   MDAY            ! day-of-month 1...28,29,30,31

!...........   SCRATCH LOCAL VARIABLES:

!        INTEGER   M, N, L
!        LOGICAL IO_360


!***********************************************************************
!   begin body of function  JULIAN

!      IO_360 = .FALSE.

!      IF (IO_360) then

!        JULIAN = MDAY + 30 * ( MNTH - 1 )

!      ELSE

!        M = MOD ((MNTH + 9), 12)
!       N = (M * 153 + 2) / 5 + MDAY + 58

!        IF      ( MOD (YEAR,   4) .NE. 0 ) THEN
!            L = 365
!        ELSE IF ( MOD (YEAR, 100) .NE. 0 ) THEN
!            L = 366
!            N = 1 + N
!        ELSE IF ( MOD (YEAR, 400) .NE. 0 )  THEN
!            L = 365
!        ELSE 
!            L = 366
!            N = 1 + N
!        END IF
!        JULIAN = 1 + MOD (N, L)
!
!      ENDIF

!      RETURN
!      END FUNCTION JULIAN 

END MODULE module_wddm
